use std::io;
use std::fs;
use std::collections::HashSet;

use crate::utils;

pub fn execute() -> Result<(), io::Error> {
    let data = fs::read_to_string("input/day06.txt")?;

    let packet_marker = find_uniq_index(&data, 4);
    let message_marker = find_uniq_index(&data, 14);

    utils::print_step(1, &packet_marker);
    utils::print_step(2, &message_marker);

    Ok(())
}

// Returns the (1-indexed) position where the first sequence of unique characters of `length`
// ends in `data`
fn find_uniq_index(data: &str, length: usize) -> usize {
    for (i, substr) in data.as_bytes().windows(length).enumerate() {
        let mut uniq = HashSet::new();
        if substr.iter().all(|x| uniq.insert(x)) {
            return i+length;
        }
    }

    panic!("No unique string of length '{length}' found.")
}
